import BigWorld
import Math
import math
import ProjectileMover
import BattleReplay
from projectile_trajectory import computeProjectileTrajectory
from constants import SERVER_TICK_LENGTH, SHELL_TRAJECTORY_EPSILON_CLIENT  # , ARENA_GUI_TYPE
from Vehicle import Vehicle
from Avatar import PlayerAvatar
from gui.Scaleform.daapi.view.meta.CrosshairPanelContainerMeta import CrosshairPanelContainerMeta
from VehicleGunRotator import VehicleGunRotator
from gui.battle_control.controllers.consumables.ammo_ctrl import AmmoReplayPlayer
import gui.Scaleform.daapi.view.battle.shared.crosshair.plugins as plug
from gui.Scaleform.daapi.view.battle.classic.stats_exchange import FragsCollectableStats

from xfw import *
from xvm_main.python.logger import *
import xvm_main.python.config as config
from xvm import utils

VEHICLE_CLASSES = {'mediumTank': 'MT', 'lightTank': 'LT', 'heavyTank': 'HT', 'AT-SPG': 'TD', 'SPG': 'SPG'}


COLOR_PIERCING_CHANCE = {'not_pierced':    '#E82929',
                         'little_pierced': '#E1C300',
                         'great_pierced':  '#2ED12F'}


maxHealth = 0
health = None
currentDistance = None
vehicle = None
player = None
timeFlight = None
timeAIM = None
sphere = None
_explosionRadius = None
isDisplaySphere = False
isDownHotkey = True
shellType = None
cameraHeight = None
arenaPeriod = None

isNotEvent = False

def explosionRadius():
    global _explosionRadius
    if vehicle is None or not hasattr(vehicle, 'typeDescriptor'):
        return
    _type = vehicle.typeDescriptor.shot.shell.type
    _explosionRadius = _type.explosionRadius if hasattr(_type, 'explosionRadius') else None


@registerEvent(AmmoReplayPlayer, 'setGunReloadTime')
def AmmoReplayPlayer_setGunReloadTime(self, timeLeft, baseTime):
    if config.get('sight/enabled', True) and isNotEvent:
        explosionRadius()


@registerEvent(CrosshairPanelContainerMeta, 'as_setReloadingS')
def CrosshairPanelContainerMeta_as_setReloadingS(self, duration, baseTime, startTime, isReloading):
    if config.get('sight/enabled', True) and isNotEvent:
        explosionRadius()


@registerEvent(FragsCollectableStats, 'addVehicleStatusUpdate')
def FragsCollectableStats_addVehicleStatusUpdate(self, vInfoVO):
    if config.get('sight/enabled', True) and isNotEvent:
        if (not vInfoVO.isAlive()) and (vehicle is not None) and (vehicle.id == vInfoVO.vehicleID):
            setValueDead()


@registerEvent(PlayerAvatar, 'handleKey')
def handleKey(self, isDown, key, mods):
    if config.get('sight/enabled', True) and isNotEvent:
        global isDownHotkey
        hotkey = config.get('sight/sphereDispersion/hotkey', None)
        if hotkey is not None and hotkey['enabled'] and (key == hotkey['keyCode']):
            if isDown:
                if hotkey['onHold']:
                    if not isDownHotkey:
                        isDownHotkey = True
                else:
                    isDownHotkey = not isDownHotkey
            else:
                if hotkey['onHold']:
                    if isDownHotkey:
                        isDownHotkey = False


@overrideMethod(plug, '_makeSettingsVO')
def plugins_makeSettingsVO(base, settingsCore, *keys):
    data = base(settingsCore, *keys)
    if config.get('sight/enabled', True) and isNotEvent:
        for mode in data:
            if config.get('sight/removeCentralMarker', False) and ('centerAlphaValue' in data[mode]):
                data[mode]['centerAlphaValue'] = 0
            if config.get('sight/removeIndicator', False) and ('netAlphaValue' in data[mode]):
                data[mode]['netAlphaValue'] = 0
            if config.get('sight/removeLoad', False) and ('reloaderAlphaValue' in data[mode]):
                data[mode]['reloaderAlphaValue'] = 0
            if config.get('sight/removeCondition', False) and ('conditionAlphaValue' in data[mode]):
                data[mode]['conditionAlphaValue'] = 0
            if config.get('sight/removeContainers', False) and ('cassetteAlphaValue' in data[mode]):
                data[mode]['cassetteAlphaValue'] = 0
            if config.get('sight/removeLoadingTimer', False) and ('reloaderTimerAlphaValue' in data[mode]):
                data[mode]['reloaderTimerAlphaValue'] = 0
            if config.get('sight/removeZoomIndicator', False) and ('zoomIndicatorAlphaValue' in data[mode]):
                data[mode]['zoomIndicatorAlphaValue'] = 0
    return data


@registerEvent(PlayerAvatar, 'onBecomePlayer')
def PlayerAvatar_onBecomePlayer(self):
    global player, arenaPeriod
    player = BigWorld.player()
    arenaPeriod = player.guiSessionProvider.shared.arenaPeriod


@registerEvent(Vehicle, 'onEnterWorld')
def Vehicle_onEnterWorld(self, prereqs):
    if self.isPlayerVehicle and config.get('sight/enabled', True):
        global maxHealth, health, isNotEvent, vehicle, currentDistance, timeFlight, timeAIM, cameraHeight
        global _explosionRadius, isDisplaySphere, isDownHotkey, player, arenaPeriod
        _explosionRadius = None
        currentDistance = None
        timeFlight = None
        timeAIM = None
        cameraHeight = None
        player = BigWorld.player()
        arenaPeriod = player.guiSessionProvider.shared.arenaPeriod
        # isNotEvent = player.arenaGuiType not in [ARENA_GUI_TYPE.EVENT_BATTLES, ARENA_GUI_TYPE.EVENT_BATTLES_2]
        isNotEvent = True
        if isNotEvent:
            vehicle = self
            td = self.typeDescriptor
            maxHealth = td.maxHealth
            health = self.health
            _type = td.type
            vehClass = VEHICLE_CLASSES[list(_type.tags.intersection(VEHICLE_CLASSES.keys()))[0]]
            isDisplaySphere = config.get('sight/sphereDispersion/enabled', False) and (vehClass in config.get('sight/sphereDispersion/vtype', ['SPG']))
            isDownHotkey = not config.get('sight/sphereDispersion/hotkey/enabled', False)


def setValueDead():
    global maxHealth, health
    global currentDistance, timeFlight, timeAIM
    global _explosionRadius, isDisplaySphere, isDownHotkey
    global cameraHeight
    _explosionRadius = None
    maxHealth = 0
    health = None
    currentDistance = None
    timeFlight = None
    timeAIM = None
    isDisplaySphere = False
    isDownHotkey = False
    cameraHeight = None
    as_event('ON_PLAYER_HEALTH')
    as_event('ON_MARKER_POSITION')
    as_event('ON_CALC_ARMOR')


@registerEvent(Vehicle, 'onHealthChanged')
def onHealthChanged(self, newHealth, attackerID, attackReasonID):
    if self.isPlayerVehicle and config.get('sight/enabled', True) and isNotEvent:
        global health
        isAlive = (newHealth > 0) and bool(vehicle.isCrewActive)
        if not isAlive:
            # setValueDead()
            if sphere in BigWorld.models():
                BigWorld.delModel(sphere)
        else:
            health = max(0, newHealth)
        as_event('ON_PLAYER_HEALTH')


def update_sphere(position):
    global sphere
    if sphere is None:
        sphere = BigWorld.Model('objects/misc/bbox/sphere1.model')
    elif sphere in BigWorld.models():
        BigWorld.delModel(sphere)
    if (_explosionRadius is not None) and (health > 0) and isDownHotkey:
        sphere.position = position
        sphere.scale = Math.Vector3(_explosionRadius, _explosionRadius, _explosionRadius)
        BigWorld.addModel(sphere)


@overrideMethod(VehicleGunRotator, '_VehicleGunRotator__getGunMarkerPosition')
def _VehicleGunRotator__getGunMarkerPosition(base, self, shotPos, shotVec, dispersionAngles):
    if config.get('sight/enabled', True) and isNotEvent:
        try:
            global timeFlight, currentDistance, timeAIM, cameraHeight
            shotDescr = self._VehicleGunRotator__avatar.getVehicleDescriptor().shot
            gravity = Math.Vector3(0.0, -shotDescr.gravity, 0.0)
            maxDist = shotDescr.maxDistance
            testVehicleID = self.getAttachedVehicleID()
            collideVehiclesAndStaticScene = ProjectileMover.collideDynamicAndStatic
            collideWithSpaceBB = self._VehicleGunRotator__avatar.arena.collideWithSpaceBB
            prevPos = shotPos
            prevVelocity = shotVec
            dt = 0.0
            maxDistCheckFlag = False
            while True:
                dt += SERVER_TICK_LENGTH
                checkPoints = computeProjectileTrajectory(prevPos, prevVelocity, gravity, SERVER_TICK_LENGTH, SHELL_TRAJECTORY_EPSILON_CLIENT)
                prevCheckPoint = prevPos
                bBreak = False
                for curCheckPoint in checkPoints:
                    testRes = collideVehiclesAndStaticScene(prevCheckPoint, curCheckPoint, (testVehicleID,))
                    if testRes is not None:
                        collData = testRes[1]
                        if collData is not None and not collData.isVehicle():
                            collData = None
                        direction = testRes[0] - prevCheckPoint
                        endPos = testRes[0]
                        timeFlight = dt if (curCheckPoint - prevCheckPoint).length < direction.length * 2 else dt - SERVER_TICK_LENGTH
                        bBreak = True
                        break
                    pos = collideWithSpaceBB(prevCheckPoint, curCheckPoint)
                    if pos is not None:
                        collData = None
                        maxDistCheckFlag = True
                        direction = pos - prevCheckPoint
                        timeFlight = dt if (curCheckPoint - prevCheckPoint).length < direction.length * 2 else dt - SERVER_TICK_LENGTH
                        endPos = pos
                        bBreak = True
                        break
                    prevCheckPoint = curCheckPoint
                if bBreak:
                    break
                prevPos = shotPos + shotVec.scale(dt) + gravity.scale(dt * dt * 0.5)
                prevVelocity = shotVec + gravity.scale(dt)

            direction.normalise()
            cameraHeight = BigWorld.camera().position.y - endPos.y
            distance = (endPos - shotPos).length
            markerDiameter = 2.0 * distance * dispersionAngles[0]
            idealMarkerDiameter = 2.0 * distance * dispersionAngles[1]
            if maxDistCheckFlag:
                if endPos.distTo(shotPos) >= maxDist:
                    direction = endPos - shotPos
                    direction.normalise()
                    endPos = shotPos + direction.scale(maxDist)
                    distance = maxDist
                    markerDiameter = 2.0 * distance * dispersionAngles[0]
                    idealMarkerDiameter = 2.0 * distance * dispersionAngles[1]
            replayCtrl = BattleReplay.g_replayCtrl
            if replayCtrl.isPlaying and replayCtrl.isClientReady:
                markerDiameter, endPos, direction = replayCtrl.getGunMarkerParams(endPos, direction)
            currentDistance = distance
            aimingInfo = player._PlayerAvatar__aimingInfo
            aimingStartTime = aimingInfo[0]
            aimingFactor = aimingInfo[1]
            shotDispMultiplierFactor = aimingInfo[2]
            # unShotDispersionFactorsTurretRotation = aimingInfo[3]
            # chassisShotDispersionFactorsMovement = aimingInfo[4]
            # chassisShotDispersionFactorsRotation = aimingInfo[5]
            # chassisShotDispersionFactorsRotation = aimingInfo[5]
            aimingTime = aimingInfo[6]
            aimingTimeAll = math.log(aimingFactor / shotDispMultiplierFactor) * aimingTime
            aimingFinishTime = aimingTimeAll + aimingStartTime
            timeAIM = max(0.0, aimingFinishTime - BigWorld.time())
            as_event('ON_MARKER_POSITION')
            if isDisplaySphere:
                update_sphere(endPos)
            return endPos, direction, markerDiameter, idealMarkerDiameter, collData
        except Exception as ex:
            err(traceback.format_exc())
            return base(self, shotPos, shotVec, dispersionAngles)
    else:
        return base(self, shotPos, shotVec, dispersionAngles)


# @registerEvent(PlayerAvatar, '_PlayerAvatar__onArenaPeriodChange')
# def __onArenaPeriodChange(self, period, periodEndTime, periodLength, periodAdditionalInfo):
#     log('period = %s, periodEndTime = %s, periodLength = %s, periodAdditionalInfo = %s' % (period, periodEndTime, periodLength, periodAdditionalInfo))


@xvm.export('sight.health', deterministic=False)
def sight_health(_health=None):
    if (health is not None) and (health > 0):
        return health if _health is None or (maxHealth == 0) else int(_health * health // maxHealth)


@xvm.export('sight.distance', deterministic=False)
def sight_distance():
    return currentDistance


@xvm.export('sight.timeFlight', deterministic=False)
def sight_timeFlight():
    return timeFlight


@xvm.export('sight.timeAIM', deterministic=False)
def sight_timeAIM():
    return timeAIM


@xvm.export('sight.cameraHeight', deterministic=False)
def sight_cameraHeight():
    return cameraHeight


@xvm.export('sight.isBattle', deterministic=False)
def sight_isBattle():
    startBattle = arenaPeriod.getPeriod() if arenaPeriod is not None else 0
    # log('startBattle = %s' % startBattle)
    return 'battle' if startBattle >= 3 else None


@xvm.export('sight.dynamic_colorRGB')
def smooth_transition_colorRGB(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('RGB', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorRBG')
def smooth_transition_colorRBG(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('RBG', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorGRB')
def smooth_transition_colorGRB(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('GRB', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorGBR')
def smooth_transition_colorGBR(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('GBR', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorBRG')
def smooth_transition_colorBRG(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('BRG', color_100, color_0, percent, maximum)


@xvm.export('sight.dynamic_colorBGR')
def smooth_transition_colorBGR(color_100, color_0, percent=None, maximum=100):
    if percent is None:
        return None
    else:
        return utils.smooth_transition_color('BGR', color_100, color_0, percent, maximum)